// SPDX-FileCopyrightText: Copyright (c) 2024 Refeyn Ltd and other QuickGraphLib contributors
// QuickGraphLib contributors SPDX-License-Identifier: MIT

#include "ColorMaps.hpp"

#include <QColor>
#include <map>

/*!
    \qmltype ColorMaps
    \inqmlmodule QuickGraphLib
    \inherits QObject
    \brief Utility functions for color maps.
*/

extern const std::map<ColorMaps::ColorMapName, QList<QRgb>> DEFAULT_COLORMAPS;

QList<QRgb> colors(ColorMaps::ColorMapName colorMapName) {
    auto iter = DEFAULT_COLORMAPS.find(colorMapName);
    if (iter != DEFAULT_COLORMAPS.end()) {
        return iter->second;
    }
    return {};
}

QList<QColor> ColorMaps::colors(ColorMaps::ColorMapName colorMapName) const {
    /*!
        \qmlmethod list<color> ColorMaps::colors(enumeration colorMapName)

        Returns the list of colors that are used in \a colorMapName. These colors are equally spaced over the range of
        values.

        \a colorMapName can take one of the following options:

            \value ColorMaps.Cividis Monotonically increasing colormap (blue/brown/yellow)
            \value ColorMaps.Grayscale Monotonically increasing colormap (black/white)
            \value ColorMaps.Inferno Monotonically increasing colormap (black/red/yellow)
            \value ColorMaps.Magma Monotonically increasing colormap (blue/purple/red/yellow)
            \value ColorMaps.Plasma Monotonically increasing colormap (black/purple/red/yellow)
            \value ColorMaps.Turbo Multihue colormap (blue/green/yellow/red)
            \value ColorMaps.Twilight Cylical colormap (white/blue/black/red/white)
            \value ColorMaps.TwilightShifted Cylical colormap (black/blue/white/red/black)
            \value ColorMaps.Viridis Monotonically increasing colormap (purple/green/yellow)

        More information about these colormaps can be found at in the
        \l {https://matplotlib.org/stable/users/explain/colors/colormaps.html} {matplotlib documentation}.

        \image examples/BuiltinColormaps.png
     */
    auto rgbs = ::colors(colorMapName);
    QList<QColor> res(rgbs.length());
    std::transform(rgbs.begin(), rgbs.end(), res.begin(), QColor::fromRgba);
    return res;
}
QString ColorMaps::colorMapName(ColorMaps::ColorMapName colorMapName) const {
    /*!
        \qmlmethod string ColorMaps::colorMapName(enumeration colorMapName)

        Returns the name of the color map \a colorMapName.

        \sa ColorMaps::colors
      */
    switch (colorMapName) {
        case ColorMaps::Grayscale:
            return "Grayscale";
        case ColorMaps::Magma:
            return "Magma";
        case ColorMaps::Inferno:
            return "Inferno";
        case ColorMaps::Plasma:
            return "Plasma";
        case ColorMaps::Viridis:
            return "Viridis";
        case ColorMaps::Cividis:
            return "Cividis";
        case ColorMaps::Twilight:
            return "Twilight";
        case ColorMaps::TwilightShifted:
            return "TwilightShifted";
        case ColorMaps::Turbo:
            return "Turbo";
    }
    return "";
}

// Numbers converted from matplotlib

const std::map<ColorMaps::ColorMapName, QList<QRgb>> DEFAULT_COLORMAPS = {
    {ColorMaps::Grayscale, {qRgb(0, 0, 0), qRgb(255, 255, 255)}},
    {ColorMaps::Magma,
     {
         qRgb(0, 0, 4),       qRgb(1, 0, 5),       qRgb(1, 1, 6),       qRgb(1, 1, 8),       qRgb(2, 1, 9),
         qRgb(2, 2, 11),      qRgb(2, 2, 13),      qRgb(3, 3, 15),      qRgb(3, 3, 18),      qRgb(4, 4, 20),
         qRgb(5, 4, 22),      qRgb(6, 5, 24),      qRgb(6, 5, 26),      qRgb(7, 6, 28),      qRgb(8, 7, 30),
         qRgb(9, 7, 32),      qRgb(10, 8, 34),     qRgb(11, 9, 36),     qRgb(12, 9, 38),     qRgb(13, 10, 41),
         qRgb(14, 11, 43),    qRgb(16, 11, 45),    qRgb(17, 12, 47),    qRgb(18, 13, 49),    qRgb(19, 13, 52),
         qRgb(20, 14, 54),    qRgb(21, 14, 56),    qRgb(22, 15, 59),    qRgb(24, 15, 61),    qRgb(25, 16, 63),
         qRgb(26, 16, 66),    qRgb(28, 16, 68),    qRgb(29, 17, 71),    qRgb(30, 17, 73),    qRgb(32, 17, 75),
         qRgb(33, 17, 78),    qRgb(34, 17, 80),    qRgb(36, 18, 83),    qRgb(37, 18, 85),    qRgb(39, 18, 88),
         qRgb(41, 17, 90),    qRgb(42, 17, 92),    qRgb(44, 17, 95),    qRgb(45, 17, 97),    qRgb(47, 17, 99),
         qRgb(49, 17, 101),   qRgb(51, 16, 103),   qRgb(52, 16, 105),   qRgb(54, 16, 107),   qRgb(56, 16, 108),
         qRgb(57, 15, 110),   qRgb(59, 15, 112),   qRgb(61, 15, 113),   qRgb(63, 15, 114),   qRgb(64, 15, 116),
         qRgb(66, 15, 117),   qRgb(68, 15, 118),   qRgb(69, 16, 119),   qRgb(71, 16, 120),   qRgb(73, 16, 120),
         qRgb(74, 16, 121),   qRgb(76, 17, 122),   qRgb(78, 17, 123),   qRgb(79, 18, 123),   qRgb(81, 18, 124),
         qRgb(82, 19, 124),   qRgb(84, 19, 125),   qRgb(86, 20, 125),   qRgb(87, 21, 126),   qRgb(89, 21, 126),
         qRgb(90, 22, 126),   qRgb(92, 22, 127),   qRgb(93, 23, 127),   qRgb(95, 24, 127),   qRgb(96, 24, 128),
         qRgb(98, 25, 128),   qRgb(100, 26, 128),  qRgb(101, 26, 128),  qRgb(103, 27, 128),  qRgb(104, 28, 129),
         qRgb(106, 28, 129),  qRgb(107, 29, 129),  qRgb(109, 29, 129),  qRgb(110, 30, 129),  qRgb(112, 31, 129),
         qRgb(114, 31, 129),  qRgb(115, 32, 129),  qRgb(117, 33, 129),  qRgb(118, 33, 129),  qRgb(120, 34, 129),
         qRgb(121, 34, 130),  qRgb(123, 35, 130),  qRgb(124, 35, 130),  qRgb(126, 36, 130),  qRgb(128, 37, 130),
         qRgb(129, 37, 129),  qRgb(131, 38, 129),  qRgb(132, 38, 129),  qRgb(134, 39, 129),  qRgb(136, 39, 129),
         qRgb(137, 40, 129),  qRgb(139, 41, 129),  qRgb(140, 41, 129),  qRgb(142, 42, 129),  qRgb(144, 42, 129),
         qRgb(145, 43, 129),  qRgb(147, 43, 128),  qRgb(148, 44, 128),  qRgb(150, 44, 128),  qRgb(152, 45, 128),
         qRgb(153, 45, 128),  qRgb(155, 46, 127),  qRgb(156, 46, 127),  qRgb(158, 47, 127),  qRgb(160, 47, 127),
         qRgb(161, 48, 126),  qRgb(163, 48, 126),  qRgb(165, 49, 126),  qRgb(166, 49, 125),  qRgb(168, 50, 125),
         qRgb(170, 51, 125),  qRgb(171, 51, 124),  qRgb(173, 52, 124),  qRgb(174, 52, 123),  qRgb(176, 53, 123),
         qRgb(178, 53, 123),  qRgb(179, 54, 122),  qRgb(181, 54, 122),  qRgb(183, 55, 121),  qRgb(184, 55, 121),
         qRgb(186, 56, 120),  qRgb(188, 57, 120),  qRgb(189, 57, 119),  qRgb(191, 58, 119),  qRgb(192, 58, 118),
         qRgb(194, 59, 117),  qRgb(196, 60, 117),  qRgb(197, 60, 116),  qRgb(199, 61, 115),  qRgb(200, 62, 115),
         qRgb(202, 62, 114),  qRgb(204, 63, 113),  qRgb(205, 64, 113),  qRgb(207, 64, 112),  qRgb(208, 65, 111),
         qRgb(210, 66, 111),  qRgb(211, 67, 110),  qRgb(213, 68, 109),  qRgb(214, 69, 108),  qRgb(216, 69, 108),
         qRgb(217, 70, 107),  qRgb(219, 71, 106),  qRgb(220, 72, 105),  qRgb(222, 73, 104),  qRgb(223, 74, 104),
         qRgb(224, 76, 103),  qRgb(226, 77, 102),  qRgb(227, 78, 101),  qRgb(228, 79, 100),  qRgb(229, 80, 100),
         qRgb(231, 82, 99),   qRgb(232, 83, 98),   qRgb(233, 84, 98),   qRgb(234, 86, 97),   qRgb(235, 87, 96),
         qRgb(236, 88, 96),   qRgb(237, 90, 95),   qRgb(238, 91, 94),   qRgb(239, 93, 94),   qRgb(240, 95, 94),
         qRgb(241, 96, 93),   qRgb(242, 98, 93),   qRgb(242, 100, 92),  qRgb(243, 101, 92),  qRgb(244, 103, 92),
         qRgb(244, 105, 92),  qRgb(245, 107, 92),  qRgb(246, 108, 92),  qRgb(246, 110, 92),  qRgb(247, 112, 92),
         qRgb(247, 114, 92),  qRgb(248, 116, 92),  qRgb(248, 118, 92),  qRgb(249, 120, 93),  qRgb(249, 121, 93),
         qRgb(249, 123, 93),  qRgb(250, 125, 94),  qRgb(250, 127, 94),  qRgb(250, 129, 95),  qRgb(251, 131, 95),
         qRgb(251, 133, 96),  qRgb(251, 135, 97),  qRgb(252, 137, 97),  qRgb(252, 138, 98),  qRgb(252, 140, 99),
         qRgb(252, 142, 100), qRgb(252, 144, 101), qRgb(253, 146, 102), qRgb(253, 148, 103), qRgb(253, 150, 104),
         qRgb(253, 152, 105), qRgb(253, 154, 106), qRgb(253, 155, 107), qRgb(254, 157, 108), qRgb(254, 159, 109),
         qRgb(254, 161, 110), qRgb(254, 163, 111), qRgb(254, 165, 113), qRgb(254, 167, 114), qRgb(254, 169, 115),
         qRgb(254, 170, 116), qRgb(254, 172, 118), qRgb(254, 174, 119), qRgb(254, 176, 120), qRgb(254, 178, 122),
         qRgb(254, 180, 123), qRgb(254, 182, 124), qRgb(254, 183, 126), qRgb(254, 185, 127), qRgb(254, 187, 129),
         qRgb(254, 189, 130), qRgb(254, 191, 132), qRgb(254, 193, 133), qRgb(254, 194, 135), qRgb(254, 196, 136),
         qRgb(254, 198, 138), qRgb(254, 200, 140), qRgb(254, 202, 141), qRgb(254, 204, 143), qRgb(254, 205, 144),
         qRgb(254, 207, 146), qRgb(254, 209, 148), qRgb(254, 211, 149), qRgb(254, 213, 151), qRgb(254, 215, 153),
         qRgb(254, 216, 154), qRgb(253, 218, 156), qRgb(253, 220, 158), qRgb(253, 222, 160), qRgb(253, 224, 161),
         qRgb(253, 226, 163), qRgb(253, 227, 165), qRgb(253, 229, 167), qRgb(253, 231, 169), qRgb(253, 233, 170),
         qRgb(253, 235, 172), qRgb(252, 236, 174), qRgb(252, 238, 176), qRgb(252, 240, 178), qRgb(252, 242, 180),
         qRgb(252, 244, 182), qRgb(252, 246, 184), qRgb(252, 247, 185), qRgb(252, 249, 187), qRgb(252, 251, 189),
         qRgb(252, 253, 191),
     }},
    {ColorMaps::Inferno,
     {
         qRgb(0, 0, 4),       qRgb(1, 0, 5),       qRgb(1, 1, 6),       qRgb(1, 1, 8),       qRgb(2, 1, 10),
         qRgb(2, 2, 12),      qRgb(2, 2, 14),      qRgb(3, 2, 16),      qRgb(4, 3, 18),      qRgb(4, 3, 20),
         qRgb(5, 4, 23),      qRgb(6, 4, 25),      qRgb(7, 5, 27),      qRgb(8, 5, 29),      qRgb(9, 6, 31),
         qRgb(10, 7, 34),     qRgb(11, 7, 36),     qRgb(12, 8, 38),     qRgb(13, 8, 41),     qRgb(14, 9, 43),
         qRgb(16, 9, 45),     qRgb(17, 10, 48),    qRgb(18, 10, 50),    qRgb(20, 11, 52),    qRgb(21, 11, 55),
         qRgb(22, 11, 57),    qRgb(24, 12, 60),    qRgb(25, 12, 62),    qRgb(27, 12, 65),    qRgb(28, 12, 67),
         qRgb(30, 12, 69),    qRgb(31, 12, 72),    qRgb(33, 12, 74),    qRgb(35, 12, 76),    qRgb(36, 12, 79),
         qRgb(38, 12, 81),    qRgb(40, 11, 83),    qRgb(41, 11, 85),    qRgb(43, 11, 87),    qRgb(45, 11, 89),
         qRgb(47, 10, 91),    qRgb(49, 10, 92),    qRgb(50, 10, 94),    qRgb(52, 10, 95),    qRgb(54, 9, 97),
         qRgb(56, 9, 98),     qRgb(57, 9, 99),     qRgb(59, 9, 100),    qRgb(61, 9, 101),    qRgb(62, 9, 102),
         qRgb(64, 10, 103),   qRgb(66, 10, 104),   qRgb(68, 10, 104),   qRgb(69, 10, 105),   qRgb(71, 11, 106),
         qRgb(73, 11, 106),   qRgb(74, 12, 107),   qRgb(76, 12, 107),   qRgb(77, 13, 108),   qRgb(79, 13, 108),
         qRgb(81, 14, 108),   qRgb(82, 14, 109),   qRgb(84, 15, 109),   qRgb(85, 15, 109),   qRgb(87, 16, 110),
         qRgb(89, 16, 110),   qRgb(90, 17, 110),   qRgb(92, 18, 110),   qRgb(93, 18, 110),   qRgb(95, 19, 110),
         qRgb(97, 19, 110),   qRgb(98, 20, 110),   qRgb(100, 21, 110),  qRgb(101, 21, 110),  qRgb(103, 22, 110),
         qRgb(105, 22, 110),  qRgb(106, 23, 110),  qRgb(108, 24, 110),  qRgb(109, 24, 110),  qRgb(111, 25, 110),
         qRgb(113, 25, 110),  qRgb(114, 26, 110),  qRgb(116, 26, 110),  qRgb(117, 27, 110),  qRgb(119, 28, 109),
         qRgb(120, 28, 109),  qRgb(122, 29, 109),  qRgb(124, 29, 109),  qRgb(125, 30, 109),  qRgb(127, 30, 108),
         qRgb(128, 31, 108),  qRgb(130, 32, 108),  qRgb(132, 32, 107),  qRgb(133, 33, 107),  qRgb(135, 33, 107),
         qRgb(136, 34, 106),  qRgb(138, 34, 106),  qRgb(140, 35, 105),  qRgb(141, 35, 105),  qRgb(143, 36, 105),
         qRgb(144, 37, 104),  qRgb(146, 37, 104),  qRgb(147, 38, 103),  qRgb(149, 38, 103),  qRgb(151, 39, 102),
         qRgb(152, 39, 102),  qRgb(154, 40, 101),  qRgb(155, 41, 100),  qRgb(157, 41, 100),  qRgb(159, 42, 99),
         qRgb(160, 42, 99),   qRgb(162, 43, 98),   qRgb(163, 44, 97),   qRgb(165, 44, 96),   qRgb(166, 45, 96),
         qRgb(168, 46, 95),   qRgb(169, 46, 94),   qRgb(171, 47, 94),   qRgb(173, 48, 93),   qRgb(174, 48, 92),
         qRgb(176, 49, 91),   qRgb(177, 50, 90),   qRgb(179, 50, 90),   qRgb(180, 51, 89),   qRgb(182, 52, 88),
         qRgb(183, 53, 87),   qRgb(185, 53, 86),   qRgb(186, 54, 85),   qRgb(188, 55, 84),   qRgb(189, 56, 83),
         qRgb(191, 57, 82),   qRgb(192, 58, 81),   qRgb(193, 58, 80),   qRgb(195, 59, 79),   qRgb(196, 60, 78),
         qRgb(198, 61, 77),   qRgb(199, 62, 76),   qRgb(200, 63, 75),   qRgb(202, 64, 74),   qRgb(203, 65, 73),
         qRgb(204, 66, 72),   qRgb(206, 67, 71),   qRgb(207, 68, 70),   qRgb(208, 69, 69),   qRgb(210, 70, 68),
         qRgb(211, 71, 67),   qRgb(212, 72, 66),   qRgb(213, 74, 65),   qRgb(215, 75, 63),   qRgb(216, 76, 62),
         qRgb(217, 77, 61),   qRgb(218, 78, 60),   qRgb(219, 80, 59),   qRgb(221, 81, 58),   qRgb(222, 82, 56),
         qRgb(223, 83, 55),   qRgb(224, 85, 54),   qRgb(225, 86, 53),   qRgb(226, 87, 52),   qRgb(227, 89, 51),
         qRgb(228, 90, 49),   qRgb(229, 92, 48),   qRgb(230, 93, 47),   qRgb(231, 94, 46),   qRgb(232, 96, 45),
         qRgb(233, 97, 43),   qRgb(234, 99, 42),   qRgb(235, 100, 41),  qRgb(235, 102, 40),  qRgb(236, 103, 38),
         qRgb(237, 105, 37),  qRgb(238, 106, 36),  qRgb(239, 108, 35),  qRgb(239, 110, 33),  qRgb(240, 111, 32),
         qRgb(241, 113, 31),  qRgb(241, 115, 29),  qRgb(242, 116, 28),  qRgb(243, 118, 27),  qRgb(243, 120, 25),
         qRgb(244, 121, 24),  qRgb(245, 123, 23),  qRgb(245, 125, 21),  qRgb(246, 126, 20),  qRgb(246, 128, 19),
         qRgb(247, 130, 18),  qRgb(247, 132, 16),  qRgb(248, 133, 15),  qRgb(248, 135, 14),  qRgb(248, 137, 12),
         qRgb(249, 139, 11),  qRgb(249, 140, 10),  qRgb(249, 142, 9),   qRgb(250, 144, 8),   qRgb(250, 146, 7),
         qRgb(250, 148, 7),   qRgb(251, 150, 6),   qRgb(251, 151, 6),   qRgb(251, 153, 6),   qRgb(251, 155, 6),
         qRgb(251, 157, 7),   qRgb(252, 159, 7),   qRgb(252, 161, 8),   qRgb(252, 163, 9),   qRgb(252, 165, 10),
         qRgb(252, 166, 12),  qRgb(252, 168, 13),  qRgb(252, 170, 15),  qRgb(252, 172, 17),  qRgb(252, 174, 18),
         qRgb(252, 176, 20),  qRgb(252, 178, 22),  qRgb(252, 180, 24),  qRgb(251, 182, 26),  qRgb(251, 184, 29),
         qRgb(251, 186, 31),  qRgb(251, 188, 33),  qRgb(251, 190, 35),  qRgb(250, 192, 38),  qRgb(250, 194, 40),
         qRgb(250, 196, 42),  qRgb(250, 198, 45),  qRgb(249, 199, 47),  qRgb(249, 201, 50),  qRgb(249, 203, 53),
         qRgb(248, 205, 55),  qRgb(248, 207, 58),  qRgb(247, 209, 61),  qRgb(247, 211, 64),  qRgb(246, 213, 67),
         qRgb(246, 215, 70),  qRgb(245, 217, 73),  qRgb(245, 219, 76),  qRgb(244, 221, 79),  qRgb(244, 223, 83),
         qRgb(244, 225, 86),  qRgb(243, 227, 90),  qRgb(243, 229, 93),  qRgb(242, 230, 97),  qRgb(242, 232, 101),
         qRgb(242, 234, 105), qRgb(241, 236, 109), qRgb(241, 237, 113), qRgb(241, 239, 117), qRgb(241, 241, 121),
         qRgb(242, 242, 125), qRgb(242, 244, 130), qRgb(243, 245, 134), qRgb(243, 246, 138), qRgb(244, 248, 142),
         qRgb(245, 249, 146), qRgb(246, 250, 150), qRgb(248, 251, 154), qRgb(249, 252, 157), qRgb(250, 253, 161),
         qRgb(252, 255, 164),
     }},
    {ColorMaps::Plasma,
     {
         qRgb(13, 8, 135),   qRgb(16, 7, 136),   qRgb(19, 7, 137),   qRgb(22, 7, 138),   qRgb(25, 6, 140),
         qRgb(27, 6, 141),   qRgb(29, 6, 142),   qRgb(32, 6, 143),   qRgb(34, 6, 144),   qRgb(36, 6, 145),
         qRgb(38, 5, 145),   qRgb(40, 5, 146),   qRgb(42, 5, 147),   qRgb(44, 5, 148),   qRgb(46, 5, 149),
         qRgb(47, 5, 150),   qRgb(49, 5, 151),   qRgb(51, 5, 151),   qRgb(53, 4, 152),   qRgb(55, 4, 153),
         qRgb(56, 4, 154),   qRgb(58, 4, 154),   qRgb(60, 4, 155),   qRgb(62, 4, 156),   qRgb(63, 4, 156),
         qRgb(65, 4, 157),   qRgb(67, 3, 158),   qRgb(68, 3, 158),   qRgb(70, 3, 159),   qRgb(72, 3, 159),
         qRgb(73, 3, 160),   qRgb(75, 3, 161),   qRgb(76, 2, 161),   qRgb(78, 2, 162),   qRgb(80, 2, 162),
         qRgb(81, 2, 163),   qRgb(83, 2, 163),   qRgb(85, 2, 164),   qRgb(86, 1, 164),   qRgb(88, 1, 164),
         qRgb(89, 1, 165),   qRgb(91, 1, 165),   qRgb(92, 1, 166),   qRgb(94, 1, 166),   qRgb(96, 1, 166),
         qRgb(97, 0, 167),   qRgb(99, 0, 167),   qRgb(100, 0, 167),  qRgb(102, 0, 167),  qRgb(103, 0, 168),
         qRgb(105, 0, 168),  qRgb(106, 0, 168),  qRgb(108, 0, 168),  qRgb(110, 0, 168),  qRgb(111, 0, 168),
         qRgb(113, 0, 168),  qRgb(114, 1, 168),  qRgb(116, 1, 168),  qRgb(117, 1, 168),  qRgb(119, 1, 168),
         qRgb(120, 1, 168),  qRgb(122, 2, 168),  qRgb(123, 2, 168),  qRgb(125, 3, 168),  qRgb(126, 3, 168),
         qRgb(128, 4, 168),  qRgb(129, 4, 167),  qRgb(131, 5, 167),  qRgb(132, 5, 167),  qRgb(134, 6, 166),
         qRgb(135, 7, 166),  qRgb(136, 8, 166),  qRgb(138, 9, 165),  qRgb(139, 10, 165), qRgb(141, 11, 165),
         qRgb(142, 12, 164), qRgb(143, 13, 164), qRgb(145, 14, 163), qRgb(146, 15, 163), qRgb(148, 16, 162),
         qRgb(149, 17, 161), qRgb(150, 19, 161), qRgb(152, 20, 160), qRgb(153, 21, 159), qRgb(154, 22, 159),
         qRgb(156, 23, 158), qRgb(157, 24, 157), qRgb(158, 25, 157), qRgb(160, 26, 156), qRgb(161, 27, 155),
         qRgb(162, 29, 154), qRgb(163, 30, 154), qRgb(165, 31, 153), qRgb(166, 32, 152), qRgb(167, 33, 151),
         qRgb(168, 34, 150), qRgb(170, 35, 149), qRgb(171, 36, 148), qRgb(172, 38, 148), qRgb(173, 39, 147),
         qRgb(174, 40, 146), qRgb(176, 41, 145), qRgb(177, 42, 144), qRgb(178, 43, 143), qRgb(179, 44, 142),
         qRgb(180, 46, 141), qRgb(181, 47, 140), qRgb(182, 48, 139), qRgb(183, 49, 138), qRgb(184, 50, 137),
         qRgb(186, 51, 136), qRgb(187, 52, 136), qRgb(188, 53, 135), qRgb(189, 55, 134), qRgb(190, 56, 133),
         qRgb(191, 57, 132), qRgb(192, 58, 131), qRgb(193, 59, 130), qRgb(194, 60, 129), qRgb(195, 61, 128),
         qRgb(196, 62, 127), qRgb(197, 64, 126), qRgb(198, 65, 125), qRgb(199, 66, 124), qRgb(200, 67, 123),
         qRgb(201, 68, 122), qRgb(202, 69, 122), qRgb(203, 70, 121), qRgb(204, 71, 120), qRgb(204, 73, 119),
         qRgb(205, 74, 118), qRgb(206, 75, 117), qRgb(207, 76, 116), qRgb(208, 77, 115), qRgb(209, 78, 114),
         qRgb(210, 79, 113), qRgb(211, 81, 113), qRgb(212, 82, 112), qRgb(213, 83, 111), qRgb(213, 84, 110),
         qRgb(214, 85, 109), qRgb(215, 86, 108), qRgb(216, 87, 107), qRgb(217, 88, 106), qRgb(218, 90, 106),
         qRgb(218, 91, 105), qRgb(219, 92, 104), qRgb(220, 93, 103), qRgb(221, 94, 102), qRgb(222, 95, 101),
         qRgb(222, 97, 100), qRgb(223, 98, 99),  qRgb(224, 99, 99),  qRgb(225, 100, 98), qRgb(226, 101, 97),
         qRgb(226, 102, 96), qRgb(227, 104, 95), qRgb(228, 105, 94), qRgb(229, 106, 93), qRgb(229, 107, 93),
         qRgb(230, 108, 92), qRgb(231, 110, 91), qRgb(231, 111, 90), qRgb(232, 112, 89), qRgb(233, 113, 88),
         qRgb(233, 114, 87), qRgb(234, 116, 87), qRgb(235, 117, 86), qRgb(235, 118, 85), qRgb(236, 119, 84),
         qRgb(237, 121, 83), qRgb(237, 122, 82), qRgb(238, 123, 81), qRgb(239, 124, 81), qRgb(239, 126, 80),
         qRgb(240, 127, 79), qRgb(240, 128, 78), qRgb(241, 129, 77), qRgb(241, 131, 76), qRgb(242, 132, 75),
         qRgb(243, 133, 75), qRgb(243, 135, 74), qRgb(244, 136, 73), qRgb(244, 137, 72), qRgb(245, 139, 71),
         qRgb(245, 140, 70), qRgb(246, 141, 69), qRgb(246, 143, 68), qRgb(247, 144, 68), qRgb(247, 145, 67),
         qRgb(247, 147, 66), qRgb(248, 148, 65), qRgb(248, 149, 64), qRgb(249, 151, 63), qRgb(249, 152, 62),
         qRgb(249, 154, 62), qRgb(250, 155, 61), qRgb(250, 156, 60), qRgb(250, 158, 59), qRgb(251, 159, 58),
         qRgb(251, 161, 57), qRgb(251, 162, 56), qRgb(252, 163, 56), qRgb(252, 165, 55), qRgb(252, 166, 54),
         qRgb(252, 168, 53), qRgb(252, 169, 52), qRgb(253, 171, 51), qRgb(253, 172, 51), qRgb(253, 174, 50),
         qRgb(253, 175, 49), qRgb(253, 177, 48), qRgb(253, 178, 47), qRgb(253, 180, 47), qRgb(253, 181, 46),
         qRgb(254, 183, 45), qRgb(254, 184, 44), qRgb(254, 186, 44), qRgb(254, 187, 43), qRgb(254, 189, 42),
         qRgb(254, 190, 42), qRgb(254, 192, 41), qRgb(253, 194, 41), qRgb(253, 195, 40), qRgb(253, 197, 39),
         qRgb(253, 198, 39), qRgb(253, 200, 39), qRgb(253, 202, 38), qRgb(253, 203, 38), qRgb(252, 205, 37),
         qRgb(252, 206, 37), qRgb(252, 208, 37), qRgb(252, 210, 37), qRgb(251, 211, 36), qRgb(251, 213, 36),
         qRgb(251, 215, 36), qRgb(250, 216, 36), qRgb(250, 218, 36), qRgb(249, 220, 36), qRgb(249, 221, 37),
         qRgb(248, 223, 37), qRgb(248, 225, 37), qRgb(247, 226, 37), qRgb(247, 228, 37), qRgb(246, 230, 38),
         qRgb(246, 232, 38), qRgb(245, 233, 38), qRgb(245, 235, 39), qRgb(244, 237, 39), qRgb(243, 238, 39),
         qRgb(243, 240, 39), qRgb(242, 242, 39), qRgb(241, 244, 38), qRgb(241, 245, 37), qRgb(240, 247, 36),
         qRgb(240, 249, 33),
     }},
    {ColorMaps::Viridis,
     {
         qRgb(68, 1, 84),    qRgb(68, 2, 86),    qRgb(69, 4, 87),    qRgb(69, 5, 89),    qRgb(70, 7, 90),
         qRgb(70, 8, 92),    qRgb(70, 10, 93),   qRgb(70, 11, 94),   qRgb(71, 13, 96),   qRgb(71, 14, 97),
         qRgb(71, 16, 99),   qRgb(71, 17, 100),  qRgb(71, 19, 101),  qRgb(72, 20, 103),  qRgb(72, 22, 104),
         qRgb(72, 23, 105),  qRgb(72, 24, 106),  qRgb(72, 26, 108),  qRgb(72, 27, 109),  qRgb(72, 28, 110),
         qRgb(72, 29, 111),  qRgb(72, 31, 112),  qRgb(72, 32, 113),  qRgb(72, 33, 115),  qRgb(72, 35, 116),
         qRgb(72, 36, 117),  qRgb(72, 37, 118),  qRgb(72, 38, 119),  qRgb(72, 40, 120),  qRgb(72, 41, 121),
         qRgb(71, 42, 122),  qRgb(71, 44, 122),  qRgb(71, 45, 123),  qRgb(71, 46, 124),  qRgb(71, 47, 125),
         qRgb(70, 48, 126),  qRgb(70, 50, 126),  qRgb(70, 51, 127),  qRgb(70, 52, 128),  qRgb(69, 53, 129),
         qRgb(69, 55, 129),  qRgb(69, 56, 130),  qRgb(68, 57, 131),  qRgb(68, 58, 131),  qRgb(68, 59, 132),
         qRgb(67, 61, 132),  qRgb(67, 62, 133),  qRgb(66, 63, 133),  qRgb(66, 64, 134),  qRgb(66, 65, 134),
         qRgb(65, 66, 135),  qRgb(65, 68, 135),  qRgb(64, 69, 136),  qRgb(64, 70, 136),  qRgb(63, 71, 136),
         qRgb(63, 72, 137),  qRgb(62, 73, 137),  qRgb(62, 74, 137),  qRgb(62, 76, 138),  qRgb(61, 77, 138),
         qRgb(61, 78, 138),  qRgb(60, 79, 138),  qRgb(60, 80, 139),  qRgb(59, 81, 139),  qRgb(59, 82, 139),
         qRgb(58, 83, 139),  qRgb(58, 84, 140),  qRgb(57, 85, 140),  qRgb(57, 86, 140),  qRgb(56, 88, 140),
         qRgb(56, 89, 140),  qRgb(55, 90, 140),  qRgb(55, 91, 141),  qRgb(54, 92, 141),  qRgb(54, 93, 141),
         qRgb(53, 94, 141),  qRgb(53, 95, 141),  qRgb(52, 96, 141),  qRgb(52, 97, 141),  qRgb(51, 98, 141),
         qRgb(51, 99, 141),  qRgb(50, 100, 142), qRgb(50, 101, 142), qRgb(49, 102, 142), qRgb(49, 103, 142),
         qRgb(49, 104, 142), qRgb(48, 105, 142), qRgb(48, 106, 142), qRgb(47, 107, 142), qRgb(47, 108, 142),
         qRgb(46, 109, 142), qRgb(46, 110, 142), qRgb(46, 111, 142), qRgb(45, 112, 142), qRgb(45, 113, 142),
         qRgb(44, 113, 142), qRgb(44, 114, 142), qRgb(44, 115, 142), qRgb(43, 116, 142), qRgb(43, 117, 142),
         qRgb(42, 118, 142), qRgb(42, 119, 142), qRgb(42, 120, 142), qRgb(41, 121, 142), qRgb(41, 122, 142),
         qRgb(41, 123, 142), qRgb(40, 124, 142), qRgb(40, 125, 142), qRgb(39, 126, 142), qRgb(39, 127, 142),
         qRgb(39, 128, 142), qRgb(38, 129, 142), qRgb(38, 130, 142), qRgb(38, 130, 142), qRgb(37, 131, 142),
         qRgb(37, 132, 142), qRgb(37, 133, 142), qRgb(36, 134, 142), qRgb(36, 135, 142), qRgb(35, 136, 142),
         qRgb(35, 137, 142), qRgb(35, 138, 141), qRgb(34, 139, 141), qRgb(34, 140, 141), qRgb(34, 141, 141),
         qRgb(33, 142, 141), qRgb(33, 143, 141), qRgb(33, 144, 141), qRgb(33, 145, 140), qRgb(32, 146, 140),
         qRgb(32, 146, 140), qRgb(32, 147, 140), qRgb(31, 148, 140), qRgb(31, 149, 139), qRgb(31, 150, 139),
         qRgb(31, 151, 139), qRgb(31, 152, 139), qRgb(31, 153, 138), qRgb(31, 154, 138), qRgb(30, 155, 138),
         qRgb(30, 156, 137), qRgb(30, 157, 137), qRgb(31, 158, 137), qRgb(31, 159, 136), qRgb(31, 160, 136),
         qRgb(31, 161, 136), qRgb(31, 161, 135), qRgb(31, 162, 135), qRgb(32, 163, 134), qRgb(32, 164, 134),
         qRgb(33, 165, 133), qRgb(33, 166, 133), qRgb(34, 167, 133), qRgb(34, 168, 132), qRgb(35, 169, 131),
         qRgb(36, 170, 131), qRgb(37, 171, 130), qRgb(37, 172, 130), qRgb(38, 173, 129), qRgb(39, 173, 129),
         qRgb(40, 174, 128), qRgb(41, 175, 127), qRgb(42, 176, 127), qRgb(44, 177, 126), qRgb(45, 178, 125),
         qRgb(46, 179, 124), qRgb(47, 180, 124), qRgb(49, 181, 123), qRgb(50, 182, 122), qRgb(52, 182, 121),
         qRgb(53, 183, 121), qRgb(55, 184, 120), qRgb(56, 185, 119), qRgb(58, 186, 118), qRgb(59, 187, 117),
         qRgb(61, 188, 116), qRgb(63, 188, 115), qRgb(64, 189, 114), qRgb(66, 190, 113), qRgb(68, 191, 112),
         qRgb(70, 192, 111), qRgb(72, 193, 110), qRgb(74, 193, 109), qRgb(76, 194, 108), qRgb(78, 195, 107),
         qRgb(80, 196, 106), qRgb(82, 197, 105), qRgb(84, 197, 104), qRgb(86, 198, 103), qRgb(88, 199, 101),
         qRgb(90, 200, 100), qRgb(92, 200, 99),  qRgb(94, 201, 98),  qRgb(96, 202, 96),  qRgb(99, 203, 95),
         qRgb(101, 203, 94), qRgb(103, 204, 92), qRgb(105, 205, 91), qRgb(108, 205, 90), qRgb(110, 206, 88),
         qRgb(112, 207, 87), qRgb(115, 208, 86), qRgb(117, 208, 84), qRgb(119, 209, 83), qRgb(122, 209, 81),
         qRgb(124, 210, 80), qRgb(127, 211, 78), qRgb(129, 211, 77), qRgb(132, 212, 75), qRgb(134, 213, 73),
         qRgb(137, 213, 72), qRgb(139, 214, 70), qRgb(142, 214, 69), qRgb(144, 215, 67), qRgb(147, 215, 65),
         qRgb(149, 216, 64), qRgb(152, 216, 62), qRgb(155, 217, 60), qRgb(157, 217, 59), qRgb(160, 218, 57),
         qRgb(162, 218, 55), qRgb(165, 219, 54), qRgb(168, 219, 52), qRgb(170, 220, 50), qRgb(173, 220, 48),
         qRgb(176, 221, 47), qRgb(178, 221, 45), qRgb(181, 222, 43), qRgb(184, 222, 41), qRgb(186, 222, 40),
         qRgb(189, 223, 38), qRgb(192, 223, 37), qRgb(194, 223, 35), qRgb(197, 224, 33), qRgb(200, 224, 32),
         qRgb(202, 225, 31), qRgb(205, 225, 29), qRgb(208, 225, 28), qRgb(210, 226, 27), qRgb(213, 226, 26),
         qRgb(216, 226, 25), qRgb(218, 227, 25), qRgb(221, 227, 24), qRgb(223, 227, 24), qRgb(226, 228, 24),
         qRgb(229, 228, 25), qRgb(231, 228, 25), qRgb(234, 229, 26), qRgb(236, 229, 27), qRgb(239, 229, 28),
         qRgb(241, 229, 29), qRgb(244, 230, 30), qRgb(246, 230, 32), qRgb(248, 230, 33), qRgb(251, 231, 35),
         qRgb(253, 231, 37),
     }},
    {ColorMaps::Cividis,
     {
         qRgb(0, 34, 78),     qRgb(0, 35, 79),     qRgb(0, 36, 81),     qRgb(0, 37, 83),     qRgb(0, 37, 84),
         qRgb(0, 38, 86),     qRgb(0, 39, 88),     qRgb(0, 40, 89),     qRgb(0, 40, 91),     qRgb(0, 41, 93),
         qRgb(0, 42, 95),     qRgb(0, 42, 97),     qRgb(0, 43, 98),     qRgb(0, 44, 100),    qRgb(0, 44, 102),
         qRgb(0, 45, 104),    qRgb(0, 46, 106),    qRgb(0, 46, 108),    qRgb(0, 47, 109),    qRgb(0, 48, 111),
         qRgb(0, 48, 112),    qRgb(0, 49, 112),    qRgb(0, 49, 113),    qRgb(1, 50, 113),    qRgb(5, 51, 113),
         qRgb(8, 51, 112),    qRgb(12, 52, 112),   qRgb(15, 53, 112),   qRgb(18, 53, 112),   qRgb(20, 54, 112),
         qRgb(22, 55, 112),   qRgb(24, 55, 111),   qRgb(26, 56, 111),   qRgb(28, 57, 111),   qRgb(30, 58, 111),
         qRgb(32, 58, 111),   qRgb(33, 59, 110),   qRgb(35, 60, 110),   qRgb(36, 60, 110),   qRgb(38, 61, 110),
         qRgb(39, 62, 110),   qRgb(41, 63, 110),   qRgb(42, 63, 109),   qRgb(43, 64, 109),   qRgb(45, 65, 109),
         qRgb(46, 65, 109),   qRgb(47, 66, 109),   qRgb(49, 67, 109),   qRgb(50, 67, 109),   qRgb(51, 68, 109),
         qRgb(52, 69, 108),   qRgb(53, 69, 108),   qRgb(54, 70, 108),   qRgb(56, 71, 108),   qRgb(57, 72, 108),
         qRgb(58, 72, 108),   qRgb(59, 73, 108),   qRgb(60, 74, 108),   qRgb(61, 74, 108),   qRgb(62, 75, 108),
         qRgb(63, 76, 108),   qRgb(64, 76, 108),   qRgb(65, 77, 108),   qRgb(66, 78, 108),   qRgb(67, 78, 108),
         qRgb(68, 79, 108),   qRgb(69, 80, 108),   qRgb(70, 81, 108),   qRgb(71, 81, 108),   qRgb(72, 82, 108),
         qRgb(73, 83, 108),   qRgb(74, 83, 108),   qRgb(75, 84, 108),   qRgb(76, 85, 108),   qRgb(77, 85, 108),
         qRgb(78, 86, 108),   qRgb(79, 87, 108),   qRgb(80, 87, 108),   qRgb(81, 88, 109),   qRgb(82, 89, 109),
         qRgb(83, 90, 109),   qRgb(84, 90, 109),   qRgb(85, 91, 109),   qRgb(85, 92, 109),   qRgb(86, 92, 109),
         qRgb(87, 93, 109),   qRgb(88, 94, 109),   qRgb(89, 94, 110),   qRgb(90, 95, 110),   qRgb(91, 96, 110),
         qRgb(92, 97, 110),   qRgb(93, 97, 110),   qRgb(94, 98, 110),   qRgb(94, 99, 111),   qRgb(95, 99, 111),
         qRgb(96, 100, 111),  qRgb(97, 101, 111),  qRgb(98, 101, 111),  qRgb(99, 102, 112),  qRgb(100, 103, 112),
         qRgb(101, 104, 112), qRgb(101, 104, 112), qRgb(102, 105, 112), qRgb(103, 106, 113), qRgb(104, 106, 113),
         qRgb(105, 107, 113), qRgb(106, 108, 113), qRgb(107, 109, 114), qRgb(108, 109, 114), qRgb(108, 110, 114),
         qRgb(109, 111, 114), qRgb(110, 111, 115), qRgb(111, 112, 115), qRgb(112, 113, 115), qRgb(113, 114, 116),
         qRgb(114, 114, 116), qRgb(114, 115, 116), qRgb(115, 116, 117), qRgb(116, 116, 117), qRgb(117, 117, 117),
         qRgb(118, 118, 118), qRgb(119, 119, 118), qRgb(119, 119, 119), qRgb(120, 120, 119), qRgb(121, 121, 119),
         qRgb(122, 122, 120), qRgb(123, 122, 120), qRgb(124, 123, 120), qRgb(125, 124, 120), qRgb(126, 124, 120),
         qRgb(126, 125, 120), qRgb(127, 126, 120), qRgb(128, 127, 120), qRgb(129, 127, 120), qRgb(130, 128, 121),
         qRgb(131, 129, 121), qRgb(132, 130, 121), qRgb(133, 130, 121), qRgb(134, 131, 121), qRgb(135, 132, 120),
         qRgb(136, 133, 120), qRgb(137, 133, 120), qRgb(138, 134, 120), qRgb(139, 135, 120), qRgb(140, 136, 120),
         qRgb(141, 136, 120), qRgb(142, 137, 120), qRgb(143, 138, 120), qRgb(144, 139, 120), qRgb(145, 139, 120),
         qRgb(146, 140, 120), qRgb(146, 141, 120), qRgb(147, 142, 120), qRgb(148, 142, 119), qRgb(149, 143, 119),
         qRgb(150, 144, 119), qRgb(151, 145, 119), qRgb(152, 146, 119), qRgb(153, 146, 119), qRgb(154, 147, 118),
         qRgb(155, 148, 118), qRgb(156, 149, 118), qRgb(157, 149, 118), qRgb(158, 150, 118), qRgb(159, 151, 117),
         qRgb(160, 152, 117), qRgb(161, 153, 117), qRgb(162, 153, 117), qRgb(163, 154, 116), qRgb(164, 155, 116),
         qRgb(165, 156, 116), qRgb(166, 156, 116), qRgb(167, 157, 115), qRgb(168, 158, 115), qRgb(169, 159, 115),
         qRgb(170, 160, 115), qRgb(171, 160, 114), qRgb(172, 161, 114), qRgb(173, 162, 114), qRgb(174, 163, 113),
         qRgb(175, 164, 113), qRgb(176, 165, 113), qRgb(177, 165, 112), qRgb(179, 166, 112), qRgb(180, 167, 111),
         qRgb(181, 168, 111), qRgb(182, 169, 111), qRgb(183, 169, 110), qRgb(184, 170, 110), qRgb(185, 171, 109),
         qRgb(186, 172, 109), qRgb(187, 173, 109), qRgb(188, 174, 108), qRgb(189, 174, 108), qRgb(190, 175, 107),
         qRgb(191, 176, 107), qRgb(192, 177, 106), qRgb(193, 178, 106), qRgb(194, 179, 105), qRgb(195, 179, 105),
         qRgb(196, 180, 104), qRgb(197, 181, 104), qRgb(198, 182, 103), qRgb(199, 183, 103), qRgb(200, 184, 102),
         qRgb(201, 185, 101), qRgb(203, 185, 101), qRgb(204, 186, 100), qRgb(205, 187, 99),  qRgb(206, 188, 99),
         qRgb(207, 189, 98),  qRgb(208, 190, 98),  qRgb(209, 191, 97),  qRgb(210, 192, 96),  qRgb(211, 192, 95),
         qRgb(212, 193, 95),  qRgb(213, 194, 94),  qRgb(214, 195, 93),  qRgb(215, 196, 92),  qRgb(217, 197, 92),
         qRgb(218, 198, 91),  qRgb(219, 199, 90),  qRgb(220, 200, 89),  qRgb(221, 200, 88),  qRgb(222, 201, 88),
         qRgb(223, 202, 87),  qRgb(224, 203, 86),  qRgb(225, 204, 85),  qRgb(226, 205, 84),  qRgb(228, 206, 83),
         qRgb(229, 207, 82),  qRgb(230, 208, 81),  qRgb(231, 209, 80),  qRgb(232, 210, 79),  qRgb(233, 211, 78),
         qRgb(234, 211, 76),  qRgb(235, 212, 75),  qRgb(237, 213, 74),  qRgb(238, 214, 73),  qRgb(239, 215, 72),
         qRgb(240, 216, 70),  qRgb(241, 217, 69),  qRgb(242, 218, 68),  qRgb(243, 219, 66),  qRgb(245, 220, 65),
         qRgb(246, 221, 63),  qRgb(247, 222, 62),  qRgb(248, 223, 60),  qRgb(249, 224, 58),  qRgb(251, 225, 56),
         qRgb(252, 226, 54),  qRgb(253, 227, 52),  qRgb(254, 228, 52),  qRgb(254, 229, 53),  qRgb(254, 230, 54),
         qRgb(254, 232, 56),
     }},
    {ColorMaps::Twilight,
     {
         qRgb(226, 217, 226), qRgb(225, 217, 226), qRgb(225, 217, 226), qRgb(224, 217, 226), qRgb(224, 217, 226),
         qRgb(223, 217, 225), qRgb(222, 217, 225), qRgb(222, 217, 225), qRgb(221, 217, 224), qRgb(220, 217, 224),
         qRgb(220, 217, 223), qRgb(219, 216, 223), qRgb(218, 216, 223), qRgb(217, 216, 222), qRgb(216, 216, 222),
         qRgb(215, 215, 221), qRgb(214, 215, 221), qRgb(213, 214, 220), qRgb(212, 214, 220), qRgb(211, 214, 219),
         qRgb(210, 213, 219), qRgb(209, 213, 218), qRgb(208, 212, 217), qRgb(206, 211, 217), qRgb(205, 211, 216),
         qRgb(204, 210, 216), qRgb(203, 210, 215), qRgb(201, 209, 215), qRgb(200, 208, 214), qRgb(199, 208, 213),
         qRgb(197, 207, 213), qRgb(196, 206, 212), qRgb(194, 206, 212), qRgb(193, 205, 211), qRgb(191, 204, 211),
         qRgb(190, 204, 210), qRgb(188, 203, 209), qRgb(187, 202, 209), qRgb(185, 201, 208), qRgb(184, 201, 208),
         qRgb(182, 200, 207), qRgb(181, 199, 207), qRgb(179, 198, 206), qRgb(178, 198, 206), qRgb(176, 197, 205),
         qRgb(175, 196, 205), qRgb(173, 195, 205), qRgb(172, 194, 204), qRgb(170, 194, 204), qRgb(169, 193, 203),
         qRgb(167, 192, 203), qRgb(166, 191, 202), qRgb(164, 190, 202), qRgb(163, 190, 202), qRgb(161, 189, 201),
         qRgb(160, 188, 201), qRgb(158, 187, 201), qRgb(157, 186, 200), qRgb(156, 185, 200), qRgb(154, 184, 200),
         qRgb(153, 184, 200), qRgb(151, 183, 199), qRgb(150, 182, 199), qRgb(149, 181, 199), qRgb(147, 180, 198),
         qRgb(146, 179, 198), qRgb(145, 178, 198), qRgb(143, 177, 198), qRgb(142, 177, 197), qRgb(141, 176, 197),
         qRgb(140, 175, 197), qRgb(138, 174, 197), qRgb(137, 173, 197), qRgb(136, 172, 196), qRgb(135, 171, 196),
         qRgb(134, 170, 196), qRgb(133, 169, 196), qRgb(132, 168, 196), qRgb(130, 167, 195), qRgb(129, 166, 195),
         qRgb(128, 165, 195), qRgb(127, 165, 195), qRgb(126, 164, 195), qRgb(125, 163, 195), qRgb(124, 162, 194),
         qRgb(123, 161, 194), qRgb(122, 160, 194), qRgb(121, 159, 194), qRgb(120, 158, 194), qRgb(119, 157, 194),
         qRgb(118, 156, 193), qRgb(118, 155, 193), qRgb(117, 154, 193), qRgb(116, 153, 193), qRgb(115, 152, 193),
         qRgb(114, 151, 193), qRgb(113, 150, 193), qRgb(113, 149, 192), qRgb(112, 148, 192), qRgb(111, 147, 192),
         qRgb(110, 146, 192), qRgb(110, 145, 192), qRgb(109, 144, 192), qRgb(108, 143, 191), qRgb(108, 142, 191),
         qRgb(107, 141, 191), qRgb(107, 140, 191), qRgb(106, 139, 191), qRgb(105, 138, 191), qRgb(105, 137, 190),
         qRgb(104, 136, 190), qRgb(104, 135, 190), qRgb(103, 134, 190), qRgb(103, 133, 190), qRgb(102, 132, 189),
         qRgb(102, 131, 189), qRgb(102, 130, 189), qRgb(101, 128, 189), qRgb(101, 127, 189), qRgb(100, 126, 188),
         qRgb(100, 125, 188), qRgb(100, 124, 188), qRgb(99, 123, 188),  qRgb(99, 122, 187),  qRgb(99, 121, 187),
         qRgb(98, 120, 187),  qRgb(98, 119, 187),  qRgb(98, 118, 186),  qRgb(98, 117, 186),  qRgb(97, 115, 186),
         qRgb(97, 114, 186),  qRgb(97, 113, 185),  qRgb(97, 112, 185),  qRgb(97, 111, 185),  qRgb(96, 110, 184),
         qRgb(96, 109, 184),  qRgb(96, 108, 184),  qRgb(96, 106, 183),  qRgb(96, 105, 183),  qRgb(96, 104, 182),
         qRgb(96, 103, 182),  qRgb(96, 102, 182),  qRgb(95, 101, 181),  qRgb(95, 100, 181),  qRgb(95, 98, 180),
         qRgb(95, 97, 180),   qRgb(95, 96, 180),   qRgb(95, 95, 179),   qRgb(95, 94, 179),   qRgb(95, 93, 178),
         qRgb(95, 91, 178),   qRgb(95, 90, 177),   qRgb(95, 89, 177),   qRgb(95, 88, 176),   qRgb(95, 87, 176),
         qRgb(95, 85, 175),   qRgb(94, 84, 174),   qRgb(94, 83, 174),   qRgb(94, 82, 173),   qRgb(94, 81, 173),
         qRgb(94, 79, 172),   qRgb(94, 78, 171),   qRgb(94, 77, 171),   qRgb(94, 76, 170),   qRgb(94, 75, 169),
         qRgb(94, 73, 169),   qRgb(94, 72, 168),   qRgb(94, 71, 167),   qRgb(94, 70, 166),   qRgb(94, 69, 166),
         qRgb(94, 67, 165),   qRgb(93, 66, 164),   qRgb(93, 65, 163),   qRgb(93, 64, 162),   qRgb(93, 62, 161),
         qRgb(93, 61, 161),   qRgb(93, 60, 160),   qRgb(93, 59, 159),   qRgb(93, 58, 158),   qRgb(92, 56, 157),
         qRgb(92, 55, 156),   qRgb(92, 54, 155),   qRgb(92, 53, 154),   qRgb(92, 52, 153),   qRgb(91, 50, 152),
         qRgb(91, 49, 150),   qRgb(91, 48, 149),   qRgb(91, 47, 148),   qRgb(90, 46, 147),   qRgb(90, 45, 146),
         qRgb(90, 43, 144),   qRgb(89, 42, 143),   qRgb(89, 41, 142),   qRgb(89, 40, 141),   qRgb(88, 39, 139),
         qRgb(88, 38, 138),   qRgb(87, 37, 136),   qRgb(87, 36, 135),   qRgb(87, 35, 133),   qRgb(86, 34, 132),
         qRgb(86, 33, 130),   qRgb(85, 32, 129),   qRgb(85, 31, 127),   qRgb(84, 30, 126),   qRgb(83, 30, 124),
         qRgb(83, 29, 122),   qRgb(82, 28, 121),   qRgb(81, 27, 119),   qRgb(81, 26, 117),   qRgb(80, 26, 116),
         qRgb(79, 25, 114),   qRgb(79, 25, 112),   qRgb(78, 24, 111),   qRgb(77, 23, 109),   qRgb(76, 23, 107),
         qRgb(76, 22, 105),   qRgb(75, 22, 104),   qRgb(74, 21, 102),   qRgb(73, 21, 100),   qRgb(72, 21, 99),
         qRgb(71, 20, 97),    qRgb(71, 20, 95),    qRgb(70, 20, 94),    qRgb(69, 19, 92),    qRgb(68, 19, 90),
         qRgb(67, 19, 89),    qRgb(66, 18, 87),    qRgb(65, 18, 86),    qRgb(65, 18, 84),    qRgb(64, 18, 83),
         qRgb(63, 18, 81),    qRgb(62, 17, 80),    qRgb(61, 17, 78),    qRgb(61, 17, 77),    qRgb(60, 17, 75),
         qRgb(59, 17, 74),    qRgb(58, 17, 73),    qRgb(58, 17, 72),    qRgb(57, 17, 70),    qRgb(56, 17, 69),
         qRgb(55, 17, 68),    qRgb(55, 17, 67),    qRgb(54, 17, 66),    qRgb(54, 17, 65),    qRgb(53, 17, 64),
         qRgb(52, 17, 63),    qRgb(52, 17, 62),    qRgb(51, 17, 61),    qRgb(51, 17, 60),    qRgb(50, 18, 59),
         qRgb(50, 18, 58),    qRgb(49, 18, 58),    qRgb(49, 19, 57),    qRgb(48, 19, 56),    qRgb(48, 20, 55),
         qRgb(47, 20, 54),    qRgb(48, 20, 55),    qRgb(49, 19, 55),    qRgb(49, 19, 55),    qRgb(50, 18, 55),
         qRgb(51, 18, 55),    qRgb(51, 18, 55),    qRgb(52, 18, 56),    qRgb(52, 18, 56),    qRgb(53, 17, 56),
         qRgb(54, 17, 56),    qRgb(54, 17, 57),    qRgb(55, 17, 57),    qRgb(56, 17, 57),    qRgb(57, 17, 58),
         qRgb(58, 17, 58),    qRgb(58, 17, 58),    qRgb(59, 17, 59),    qRgb(60, 17, 59),    qRgb(61, 17, 60),
         qRgb(62, 17, 60),    qRgb(63, 18, 61),    qRgb(64, 18, 61),    qRgb(65, 18, 61),    qRgb(66, 18, 62),
         qRgb(67, 18, 62),    qRgb(68, 18, 63),    qRgb(70, 18, 64),    qRgb(71, 19, 64),    qRgb(72, 19, 65),
         qRgb(73, 19, 65),    qRgb(74, 19, 66),    qRgb(75, 19, 66),    qRgb(77, 20, 67),    qRgb(78, 20, 67),
         qRgb(79, 20, 68),    qRgb(80, 20, 68),    qRgb(82, 21, 69),    qRgb(83, 21, 69),    qRgb(84, 21, 70),
         qRgb(86, 21, 70),    qRgb(87, 22, 71),    qRgb(88, 22, 71),    qRgb(89, 22, 72),    qRgb(91, 22, 72),
         qRgb(92, 23, 73),    qRgb(93, 23, 73),    qRgb(95, 23, 74),    qRgb(96, 24, 74),    qRgb(97, 24, 75),
         qRgb(99, 24, 75),    qRgb(100, 25, 75),   qRgb(101, 25, 76),   qRgb(103, 25, 76),   qRgb(104, 26, 77),
         qRgb(105, 26, 77),   qRgb(107, 27, 77),   qRgb(108, 27, 78),   qRgb(109, 27, 78),   qRgb(111, 28, 78),
         qRgb(112, 28, 78),   qRgb(113, 29, 79),   qRgb(114, 29, 79),   qRgb(116, 30, 79),   qRgb(117, 30, 79),
         qRgb(118, 31, 79),   qRgb(120, 31, 79),   qRgb(121, 32, 80),   qRgb(122, 32, 80),   qRgb(123, 33, 80),
         qRgb(125, 33, 80),   qRgb(126, 34, 80),   qRgb(127, 35, 80),   qRgb(128, 35, 80),   qRgb(129, 36, 80),
         qRgb(131, 37, 80),   qRgb(132, 37, 80),   qRgb(133, 38, 80),   qRgb(134, 39, 80),   qRgb(135, 39, 80),
         qRgb(136, 40, 80),   qRgb(138, 41, 80),   qRgb(139, 42, 80),   qRgb(140, 42, 80),   qRgb(141, 43, 80),
         qRgb(142, 44, 80),   qRgb(143, 45, 80),   qRgb(144, 46, 80),   qRgb(145, 47, 80),   qRgb(146, 47, 80),
         qRgb(147, 48, 80),   qRgb(148, 49, 80),   qRgb(149, 50, 80),   qRgb(150, 51, 80),   qRgb(151, 52, 80),
         qRgb(152, 53, 80),   qRgb(153, 54, 80),   qRgb(154, 55, 80),   qRgb(155, 56, 80),   qRgb(156, 57, 80),
         qRgb(157, 58, 80),   qRgb(158, 59, 80),   qRgb(159, 60, 80),   qRgb(160, 61, 80),   qRgb(160, 62, 80),
         qRgb(161, 63, 80),   qRgb(162, 64, 80),   qRgb(163, 65, 80),   qRgb(164, 66, 80),   qRgb(165, 67, 80),
         qRgb(165, 68, 80),   qRgb(166, 69, 80),   qRgb(167, 70, 80),   qRgb(168, 71, 80),   qRgb(169, 72, 80),
         qRgb(169, 73, 80),   qRgb(170, 74, 80),   qRgb(171, 75, 80),   qRgb(172, 76, 80),   qRgb(172, 77, 81),
         qRgb(173, 78, 81),   qRgb(174, 80, 81),   qRgb(175, 81, 81),   qRgb(175, 82, 81),   qRgb(176, 83, 81),
         qRgb(177, 84, 82),   qRgb(177, 85, 82),   qRgb(178, 86, 82),   qRgb(179, 87, 82),   qRgb(179, 89, 83),
         qRgb(180, 90, 83),   qRgb(181, 91, 83),   qRgb(181, 92, 84),   qRgb(182, 93, 84),   qRgb(182, 94, 84),
         qRgb(183, 95, 85),   qRgb(184, 97, 85),   qRgb(184, 98, 85),   qRgb(185, 99, 86),   qRgb(185, 100, 86),
         qRgb(186, 101, 87),  qRgb(186, 102, 87),  qRgb(187, 104, 87),  qRgb(187, 105, 88),  qRgb(188, 106, 88),
         qRgb(188, 107, 89),  qRgb(189, 108, 90),  qRgb(189, 110, 90),  qRgb(190, 111, 91),  qRgb(190, 112, 91),
         qRgb(191, 113, 92),  qRgb(191, 114, 93),  qRgb(192, 116, 93),  qRgb(192, 117, 94),  qRgb(192, 118, 95),
         qRgb(193, 119, 95),  qRgb(193, 121, 96),  qRgb(194, 122, 97),  qRgb(194, 123, 98),  qRgb(194, 124, 99),
         qRgb(195, 125, 99),  qRgb(195, 127, 100), qRgb(196, 128, 101), qRgb(196, 129, 102), qRgb(196, 130, 103),
         qRgb(197, 132, 104), qRgb(197, 133, 105), qRgb(197, 134, 106), qRgb(198, 135, 107), qRgb(198, 137, 108),
         qRgb(198, 138, 109), qRgb(198, 139, 110), qRgb(199, 140, 111), qRgb(199, 142, 113), qRgb(199, 143, 114),
         qRgb(200, 144, 115), qRgb(200, 145, 116), qRgb(200, 146, 117), qRgb(200, 148, 119), qRgb(201, 149, 120),
         qRgb(201, 150, 121), qRgb(201, 151, 123), qRgb(202, 153, 124), qRgb(202, 154, 125), qRgb(202, 155, 127),
         qRgb(202, 156, 128), qRgb(203, 157, 130), qRgb(203, 159, 131), qRgb(203, 160, 133), qRgb(204, 161, 134),
         qRgb(204, 162, 135), qRgb(204, 163, 137), qRgb(204, 165, 139), qRgb(205, 166, 140), qRgb(205, 167, 142),
         qRgb(205, 168, 143), qRgb(206, 169, 145), qRgb(206, 171, 146), qRgb(206, 172, 148), qRgb(207, 173, 150),
         qRgb(207, 174, 151), qRgb(207, 175, 153), qRgb(208, 176, 155), qRgb(208, 178, 156), qRgb(208, 179, 158),
         qRgb(209, 180, 160), qRgb(209, 181, 161), qRgb(209, 182, 163), qRgb(210, 183, 165), qRgb(210, 184, 167),
         qRgb(211, 185, 168), qRgb(211, 186, 170), qRgb(212, 188, 172), qRgb(212, 189, 173), qRgb(212, 190, 175),
         qRgb(213, 191, 177), qRgb(213, 192, 179), qRgb(214, 193, 180), qRgb(214, 194, 182), qRgb(215, 195, 184),
         qRgb(215, 196, 185), qRgb(216, 197, 187), qRgb(216, 198, 189), qRgb(216, 199, 190), qRgb(217, 200, 192),
         qRgb(217, 201, 194), qRgb(218, 202, 195), qRgb(218, 203, 197), qRgb(219, 203, 198), qRgb(219, 204, 200),
         qRgb(220, 205, 202), qRgb(220, 206, 203), qRgb(220, 207, 205), qRgb(221, 208, 206), qRgb(221, 208, 207),
         qRgb(221, 209, 209), qRgb(222, 210, 210), qRgb(222, 211, 211), qRgb(222, 211, 212), qRgb(223, 212, 214),
         qRgb(223, 212, 215), qRgb(223, 213, 216), qRgb(224, 214, 217), qRgb(224, 214, 218), qRgb(224, 215, 218),
         qRgb(224, 215, 219), qRgb(225, 215, 220), qRgb(225, 216, 221), qRgb(225, 216, 222), qRgb(225, 216, 223),
         qRgb(226, 216, 223), qRgb(226, 217, 224), qRgb(226, 217, 225), qRgb(226, 217, 225), qRgb(226, 217, 226),
     }},
    {ColorMaps::TwilightShifted,
     {
         qRgb(48, 20, 55),    qRgb(48, 19, 56),    qRgb(49, 19, 57),    qRgb(49, 18, 58),    qRgb(50, 18, 58),
         qRgb(50, 18, 59),    qRgb(51, 17, 60),    qRgb(51, 17, 61),    qRgb(52, 17, 62),    qRgb(52, 17, 63),
         qRgb(53, 17, 64),    qRgb(54, 17, 65),    qRgb(54, 17, 66),    qRgb(55, 17, 67),    qRgb(55, 17, 68),
         qRgb(56, 17, 69),    qRgb(57, 17, 70),    qRgb(58, 17, 72),    qRgb(58, 17, 73),    qRgb(59, 17, 74),
         qRgb(60, 17, 75),    qRgb(61, 17, 77),    qRgb(61, 17, 78),    qRgb(62, 17, 80),    qRgb(63, 18, 81),
         qRgb(64, 18, 83),    qRgb(65, 18, 84),    qRgb(65, 18, 86),    qRgb(66, 18, 87),    qRgb(67, 19, 89),
         qRgb(68, 19, 90),    qRgb(69, 19, 92),    qRgb(70, 20, 94),    qRgb(71, 20, 95),    qRgb(71, 20, 97),
         qRgb(72, 21, 99),    qRgb(73, 21, 100),   qRgb(74, 21, 102),   qRgb(75, 22, 104),   qRgb(76, 22, 105),
         qRgb(76, 23, 107),   qRgb(77, 23, 109),   qRgb(78, 24, 111),   qRgb(79, 25, 112),   qRgb(79, 25, 114),
         qRgb(80, 26, 116),   qRgb(81, 26, 117),   qRgb(81, 27, 119),   qRgb(82, 28, 121),   qRgb(83, 29, 122),
         qRgb(83, 30, 124),   qRgb(84, 30, 126),   qRgb(85, 31, 127),   qRgb(85, 32, 129),   qRgb(86, 33, 130),
         qRgb(86, 34, 132),   qRgb(87, 35, 133),   qRgb(87, 36, 135),   qRgb(87, 37, 136),   qRgb(88, 38, 138),
         qRgb(88, 39, 139),   qRgb(89, 40, 141),   qRgb(89, 41, 142),   qRgb(89, 42, 143),   qRgb(90, 43, 144),
         qRgb(90, 45, 146),   qRgb(90, 46, 147),   qRgb(91, 47, 148),   qRgb(91, 48, 149),   qRgb(91, 49, 150),
         qRgb(91, 50, 152),   qRgb(92, 52, 153),   qRgb(92, 53, 154),   qRgb(92, 54, 155),   qRgb(92, 55, 156),
         qRgb(92, 56, 157),   qRgb(93, 58, 158),   qRgb(93, 59, 159),   qRgb(93, 60, 160),   qRgb(93, 61, 161),
         qRgb(93, 62, 161),   qRgb(93, 64, 162),   qRgb(93, 65, 163),   qRgb(93, 66, 164),   qRgb(94, 67, 165),
         qRgb(94, 69, 166),   qRgb(94, 70, 166),   qRgb(94, 71, 167),   qRgb(94, 72, 168),   qRgb(94, 73, 169),
         qRgb(94, 75, 169),   qRgb(94, 76, 170),   qRgb(94, 77, 171),   qRgb(94, 78, 171),   qRgb(94, 79, 172),
         qRgb(94, 81, 173),   qRgb(94, 82, 173),   qRgb(94, 83, 174),   qRgb(94, 84, 174),   qRgb(95, 85, 175),
         qRgb(95, 87, 176),   qRgb(95, 88, 176),   qRgb(95, 89, 177),   qRgb(95, 90, 177),   qRgb(95, 91, 178),
         qRgb(95, 93, 178),   qRgb(95, 94, 179),   qRgb(95, 95, 179),   qRgb(95, 96, 180),   qRgb(95, 97, 180),
         qRgb(95, 98, 180),   qRgb(95, 100, 181),  qRgb(95, 101, 181),  qRgb(96, 102, 182),  qRgb(96, 103, 182),
         qRgb(96, 104, 182),  qRgb(96, 105, 183),  qRgb(96, 106, 183),  qRgb(96, 108, 184),  qRgb(96, 109, 184),
         qRgb(96, 110, 184),  qRgb(97, 111, 185),  qRgb(97, 112, 185),  qRgb(97, 113, 185),  qRgb(97, 114, 186),
         qRgb(97, 115, 186),  qRgb(98, 117, 186),  qRgb(98, 118, 186),  qRgb(98, 119, 187),  qRgb(98, 120, 187),
         qRgb(99, 121, 187),  qRgb(99, 122, 187),  qRgb(99, 123, 188),  qRgb(100, 124, 188), qRgb(100, 125, 188),
         qRgb(100, 126, 188), qRgb(101, 127, 189), qRgb(101, 128, 189), qRgb(102, 130, 189), qRgb(102, 131, 189),
         qRgb(102, 132, 189), qRgb(103, 133, 190), qRgb(103, 134, 190), qRgb(104, 135, 190), qRgb(104, 136, 190),
         qRgb(105, 137, 190), qRgb(105, 138, 191), qRgb(106, 139, 191), qRgb(107, 140, 191), qRgb(107, 141, 191),
         qRgb(108, 142, 191), qRgb(108, 143, 191), qRgb(109, 144, 192), qRgb(110, 145, 192), qRgb(110, 146, 192),
         qRgb(111, 147, 192), qRgb(112, 148, 192), qRgb(113, 149, 192), qRgb(113, 150, 193), qRgb(114, 151, 193),
         qRgb(115, 152, 193), qRgb(116, 153, 193), qRgb(117, 154, 193), qRgb(118, 155, 193), qRgb(118, 156, 193),
         qRgb(119, 157, 194), qRgb(120, 158, 194), qRgb(121, 159, 194), qRgb(122, 160, 194), qRgb(123, 161, 194),
         qRgb(124, 162, 194), qRgb(125, 163, 195), qRgb(126, 164, 195), qRgb(127, 165, 195), qRgb(128, 165, 195),
         qRgb(129, 166, 195), qRgb(130, 167, 195), qRgb(132, 168, 196), qRgb(133, 169, 196), qRgb(134, 170, 196),
         qRgb(135, 171, 196), qRgb(136, 172, 196), qRgb(137, 173, 197), qRgb(138, 174, 197), qRgb(140, 175, 197),
         qRgb(141, 176, 197), qRgb(142, 177, 197), qRgb(143, 177, 198), qRgb(145, 178, 198), qRgb(146, 179, 198),
         qRgb(147, 180, 198), qRgb(149, 181, 199), qRgb(150, 182, 199), qRgb(151, 183, 199), qRgb(153, 184, 200),
         qRgb(154, 184, 200), qRgb(156, 185, 200), qRgb(157, 186, 200), qRgb(158, 187, 201), qRgb(160, 188, 201),
         qRgb(161, 189, 201), qRgb(163, 190, 202), qRgb(164, 190, 202), qRgb(166, 191, 202), qRgb(167, 192, 203),
         qRgb(169, 193, 203), qRgb(170, 194, 204), qRgb(172, 194, 204), qRgb(173, 195, 205), qRgb(175, 196, 205),
         qRgb(176, 197, 205), qRgb(178, 198, 206), qRgb(179, 198, 206), qRgb(181, 199, 207), qRgb(182, 200, 207),
         qRgb(184, 201, 208), qRgb(185, 201, 208), qRgb(187, 202, 209), qRgb(188, 203, 209), qRgb(190, 204, 210),
         qRgb(191, 204, 211), qRgb(193, 205, 211), qRgb(194, 206, 212), qRgb(196, 206, 212), qRgb(197, 207, 213),
         qRgb(199, 208, 213), qRgb(200, 208, 214), qRgb(201, 209, 215), qRgb(203, 210, 215), qRgb(204, 210, 216),
         qRgb(205, 211, 216), qRgb(206, 211, 217), qRgb(208, 212, 217), qRgb(209, 213, 218), qRgb(210, 213, 219),
         qRgb(211, 214, 219), qRgb(212, 214, 220), qRgb(213, 214, 220), qRgb(214, 215, 221), qRgb(215, 215, 221),
         qRgb(216, 216, 222), qRgb(217, 216, 222), qRgb(218, 216, 223), qRgb(219, 216, 223), qRgb(220, 217, 223),
         qRgb(220, 217, 224), qRgb(221, 217, 224), qRgb(222, 217, 225), qRgb(222, 217, 225), qRgb(223, 217, 225),
         qRgb(224, 217, 226), qRgb(224, 217, 226), qRgb(225, 217, 226), qRgb(225, 217, 226), qRgb(226, 217, 226),
         qRgb(226, 217, 226), qRgb(226, 217, 225), qRgb(226, 217, 225), qRgb(226, 217, 224), qRgb(226, 216, 223),
         qRgb(225, 216, 223), qRgb(225, 216, 222), qRgb(225, 216, 221), qRgb(225, 215, 220), qRgb(224, 215, 219),
         qRgb(224, 215, 218), qRgb(224, 214, 218), qRgb(224, 214, 217), qRgb(223, 213, 216), qRgb(223, 212, 215),
         qRgb(223, 212, 214), qRgb(222, 211, 212), qRgb(222, 211, 211), qRgb(222, 210, 210), qRgb(221, 209, 209),
         qRgb(221, 208, 207), qRgb(221, 208, 206), qRgb(220, 207, 205), qRgb(220, 206, 203), qRgb(220, 205, 202),
         qRgb(219, 204, 200), qRgb(219, 203, 198), qRgb(218, 203, 197), qRgb(218, 202, 195), qRgb(217, 201, 194),
         qRgb(217, 200, 192), qRgb(216, 199, 190), qRgb(216, 198, 189), qRgb(216, 197, 187), qRgb(215, 196, 185),
         qRgb(215, 195, 184), qRgb(214, 194, 182), qRgb(214, 193, 180), qRgb(213, 192, 179), qRgb(213, 191, 177),
         qRgb(212, 190, 175), qRgb(212, 189, 173), qRgb(212, 188, 172), qRgb(211, 186, 170), qRgb(211, 185, 168),
         qRgb(210, 184, 167), qRgb(210, 183, 165), qRgb(209, 182, 163), qRgb(209, 181, 161), qRgb(209, 180, 160),
         qRgb(208, 179, 158), qRgb(208, 178, 156), qRgb(208, 176, 155), qRgb(207, 175, 153), qRgb(207, 174, 151),
         qRgb(207, 173, 150), qRgb(206, 172, 148), qRgb(206, 171, 146), qRgb(206, 169, 145), qRgb(205, 168, 143),
         qRgb(205, 167, 142), qRgb(205, 166, 140), qRgb(204, 165, 139), qRgb(204, 163, 137), qRgb(204, 162, 135),
         qRgb(204, 161, 134), qRgb(203, 160, 133), qRgb(203, 159, 131), qRgb(203, 157, 130), qRgb(202, 156, 128),
         qRgb(202, 155, 127), qRgb(202, 154, 125), qRgb(202, 153, 124), qRgb(201, 151, 123), qRgb(201, 150, 121),
         qRgb(201, 149, 120), qRgb(200, 148, 119), qRgb(200, 146, 117), qRgb(200, 145, 116), qRgb(200, 144, 115),
         qRgb(199, 143, 114), qRgb(199, 142, 113), qRgb(199, 140, 111), qRgb(198, 139, 110), qRgb(198, 138, 109),
         qRgb(198, 137, 108), qRgb(198, 135, 107), qRgb(197, 134, 106), qRgb(197, 133, 105), qRgb(197, 132, 104),
         qRgb(196, 130, 103), qRgb(196, 129, 102), qRgb(196, 128, 101), qRgb(195, 127, 100), qRgb(195, 125, 99),
         qRgb(194, 124, 99),  qRgb(194, 123, 98),  qRgb(194, 122, 97),  qRgb(193, 121, 96),  qRgb(193, 119, 95),
         qRgb(192, 118, 95),  qRgb(192, 117, 94),  qRgb(192, 116, 93),  qRgb(191, 114, 93),  qRgb(191, 113, 92),
         qRgb(190, 112, 91),  qRgb(190, 111, 91),  qRgb(189, 110, 90),  qRgb(189, 108, 90),  qRgb(188, 107, 89),
         qRgb(188, 106, 88),  qRgb(187, 105, 88),  qRgb(187, 104, 87),  qRgb(186, 102, 87),  qRgb(186, 101, 87),
         qRgb(185, 100, 86),  qRgb(185, 99, 86),   qRgb(184, 98, 85),   qRgb(184, 97, 85),   qRgb(183, 95, 85),
         qRgb(182, 94, 84),   qRgb(182, 93, 84),   qRgb(181, 92, 84),   qRgb(181, 91, 83),   qRgb(180, 90, 83),
         qRgb(179, 89, 83),   qRgb(179, 87, 82),   qRgb(178, 86, 82),   qRgb(177, 85, 82),   qRgb(177, 84, 82),
         qRgb(176, 83, 81),   qRgb(175, 82, 81),   qRgb(175, 81, 81),   qRgb(174, 80, 81),   qRgb(173, 78, 81),
         qRgb(172, 77, 81),   qRgb(172, 76, 80),   qRgb(171, 75, 80),   qRgb(170, 74, 80),   qRgb(169, 73, 80),
         qRgb(169, 72, 80),   qRgb(168, 71, 80),   qRgb(167, 70, 80),   qRgb(166, 69, 80),   qRgb(165, 68, 80),
         qRgb(165, 67, 80),   qRgb(164, 66, 80),   qRgb(163, 65, 80),   qRgb(162, 64, 80),   qRgb(161, 63, 80),
         qRgb(160, 62, 80),   qRgb(160, 61, 80),   qRgb(159, 60, 80),   qRgb(158, 59, 80),   qRgb(157, 58, 80),
         qRgb(156, 57, 80),   qRgb(155, 56, 80),   qRgb(154, 55, 80),   qRgb(153, 54, 80),   qRgb(152, 53, 80),
         qRgb(151, 52, 80),   qRgb(150, 51, 80),   qRgb(149, 50, 80),   qRgb(148, 49, 80),   qRgb(147, 48, 80),
         qRgb(146, 47, 80),   qRgb(145, 47, 80),   qRgb(144, 46, 80),   qRgb(143, 45, 80),   qRgb(142, 44, 80),
         qRgb(141, 43, 80),   qRgb(140, 42, 80),   qRgb(139, 42, 80),   qRgb(138, 41, 80),   qRgb(136, 40, 80),
         qRgb(135, 39, 80),   qRgb(134, 39, 80),   qRgb(133, 38, 80),   qRgb(132, 37, 80),   qRgb(131, 37, 80),
         qRgb(129, 36, 80),   qRgb(128, 35, 80),   qRgb(127, 35, 80),   qRgb(126, 34, 80),   qRgb(125, 33, 80),
         qRgb(123, 33, 80),   qRgb(122, 32, 80),   qRgb(121, 32, 80),   qRgb(120, 31, 79),   qRgb(118, 31, 79),
         qRgb(117, 30, 79),   qRgb(116, 30, 79),   qRgb(114, 29, 79),   qRgb(113, 29, 79),   qRgb(112, 28, 78),
         qRgb(111, 28, 78),   qRgb(109, 27, 78),   qRgb(108, 27, 78),   qRgb(107, 27, 77),   qRgb(105, 26, 77),
         qRgb(104, 26, 77),   qRgb(103, 25, 76),   qRgb(101, 25, 76),   qRgb(100, 25, 75),   qRgb(99, 24, 75),
         qRgb(97, 24, 75),    qRgb(96, 24, 74),    qRgb(95, 23, 74),    qRgb(93, 23, 73),    qRgb(92, 23, 73),
         qRgb(91, 22, 72),    qRgb(89, 22, 72),    qRgb(88, 22, 71),    qRgb(87, 22, 71),    qRgb(86, 21, 70),
         qRgb(84, 21, 70),    qRgb(83, 21, 69),    qRgb(82, 21, 69),    qRgb(80, 20, 68),    qRgb(79, 20, 68),
         qRgb(78, 20, 67),    qRgb(77, 20, 67),    qRgb(75, 19, 66),    qRgb(74, 19, 66),    qRgb(73, 19, 65),
         qRgb(72, 19, 65),    qRgb(71, 19, 64),    qRgb(70, 18, 64),    qRgb(68, 18, 63),    qRgb(67, 18, 62),
         qRgb(66, 18, 62),    qRgb(65, 18, 61),    qRgb(64, 18, 61),    qRgb(63, 18, 61),    qRgb(62, 17, 60),
         qRgb(61, 17, 60),    qRgb(60, 17, 59),    qRgb(59, 17, 59),    qRgb(58, 17, 58),    qRgb(58, 17, 58),
         qRgb(57, 17, 58),    qRgb(56, 17, 57),    qRgb(55, 17, 57),    qRgb(54, 17, 57),    qRgb(54, 17, 56),
         qRgb(53, 17, 56),    qRgb(52, 18, 56),    qRgb(52, 18, 56),    qRgb(51, 18, 55),    qRgb(51, 18, 55),
         qRgb(50, 18, 55),    qRgb(49, 19, 55),    qRgb(49, 19, 55),    qRgb(48, 20, 55),    qRgb(47, 20, 54),
     }},
    {ColorMaps::Turbo,
     {
         qRgb(48, 18, 59),    qRgb(50, 21, 67),    qRgb(51, 24, 74),   qRgb(52, 27, 81),   qRgb(53, 30, 88),
         qRgb(54, 33, 95),    qRgb(55, 36, 102),   qRgb(56, 39, 109),  qRgb(57, 42, 115),  qRgb(58, 45, 121),
         qRgb(59, 47, 128),   qRgb(60, 50, 134),   qRgb(61, 53, 139),  qRgb(62, 56, 145),  qRgb(63, 59, 151),
         qRgb(63, 62, 156),   qRgb(64, 64, 162),   qRgb(65, 67, 167),  qRgb(65, 70, 172),  qRgb(66, 73, 177),
         qRgb(66, 75, 181),   qRgb(67, 78, 186),   qRgb(68, 81, 191),  qRgb(68, 84, 195),  qRgb(68, 86, 199),
         qRgb(69, 89, 203),   qRgb(69, 92, 207),   qRgb(69, 94, 211),  qRgb(70, 97, 214),  qRgb(70, 100, 218),
         qRgb(70, 102, 221),  qRgb(70, 105, 224),  qRgb(70, 107, 227), qRgb(71, 110, 230), qRgb(71, 113, 233),
         qRgb(71, 115, 235),  qRgb(71, 118, 238),  qRgb(71, 120, 240), qRgb(71, 123, 242), qRgb(70, 125, 244),
         qRgb(70, 128, 246),  qRgb(70, 130, 248),  qRgb(70, 133, 250), qRgb(70, 135, 251), qRgb(69, 138, 252),
         qRgb(69, 140, 253),  qRgb(68, 143, 254),  qRgb(67, 145, 254), qRgb(66, 148, 255), qRgb(65, 150, 255),
         qRgb(64, 153, 255),  qRgb(62, 155, 254),  qRgb(61, 158, 254), qRgb(59, 160, 253), qRgb(58, 163, 252),
         qRgb(56, 165, 251),  qRgb(55, 168, 250),  qRgb(53, 171, 248), qRgb(51, 173, 247), qRgb(49, 175, 245),
         qRgb(47, 178, 244),  qRgb(46, 180, 242),  qRgb(44, 183, 240), qRgb(42, 185, 238), qRgb(40, 188, 235),
         qRgb(39, 190, 233),  qRgb(37, 192, 231),  qRgb(35, 195, 228), qRgb(34, 197, 226), qRgb(32, 199, 223),
         qRgb(31, 201, 221),  qRgb(30, 203, 218),  qRgb(28, 205, 216), qRgb(27, 208, 213), qRgb(26, 210, 210),
         qRgb(26, 212, 208),  qRgb(25, 213, 205),  qRgb(24, 215, 202), qRgb(24, 217, 200), qRgb(24, 219, 197),
         qRgb(24, 221, 194),  qRgb(24, 222, 192),  qRgb(24, 224, 189), qRgb(25, 226, 187), qRgb(25, 227, 185),
         qRgb(26, 228, 182),  qRgb(28, 230, 180),  qRgb(29, 231, 178), qRgb(31, 233, 175), qRgb(32, 234, 172),
         qRgb(34, 235, 170),  qRgb(37, 236, 167),  qRgb(39, 238, 164), qRgb(42, 239, 161), qRgb(44, 240, 158),
         qRgb(47, 241, 155),  qRgb(50, 242, 152),  qRgb(53, 243, 148), qRgb(56, 244, 145), qRgb(60, 245, 142),
         qRgb(63, 246, 138),  qRgb(67, 247, 135),  qRgb(70, 248, 132), qRgb(74, 248, 128), qRgb(78, 249, 125),
         qRgb(82, 250, 122),  qRgb(85, 250, 118),  qRgb(89, 251, 115), qRgb(93, 252, 111), qRgb(97, 252, 108),
         qRgb(101, 253, 105), qRgb(105, 253, 102), qRgb(109, 254, 98), qRgb(113, 254, 95), qRgb(117, 254, 92),
         qRgb(121, 254, 89),  qRgb(125, 255, 86),  qRgb(128, 255, 83), qRgb(132, 255, 81), qRgb(136, 255, 78),
         qRgb(139, 255, 75),  qRgb(143, 255, 73),  qRgb(146, 255, 71), qRgb(150, 254, 68), qRgb(153, 254, 66),
         qRgb(156, 254, 64),  qRgb(159, 253, 63),  qRgb(161, 253, 61), qRgb(164, 252, 60), qRgb(167, 252, 58),
         qRgb(169, 251, 57),  qRgb(172, 251, 56),  qRgb(175, 250, 55), qRgb(177, 249, 54), qRgb(180, 248, 54),
         qRgb(183, 247, 53),  qRgb(185, 246, 53),  qRgb(188, 245, 52), qRgb(190, 244, 52), qRgb(193, 243, 52),
         qRgb(195, 241, 52),  qRgb(198, 240, 52),  qRgb(200, 239, 52), qRgb(203, 237, 52), qRgb(205, 236, 52),
         qRgb(208, 234, 52),  qRgb(210, 233, 53),  qRgb(212, 231, 53), qRgb(215, 229, 53), qRgb(217, 228, 54),
         qRgb(219, 226, 54),  qRgb(221, 224, 55),  qRgb(223, 223, 55), qRgb(225, 221, 55), qRgb(227, 219, 56),
         qRgb(229, 217, 56),  qRgb(231, 215, 57),  qRgb(233, 213, 57), qRgb(235, 211, 57), qRgb(236, 209, 58),
         qRgb(238, 207, 58),  qRgb(239, 205, 58),  qRgb(241, 203, 58), qRgb(242, 201, 58), qRgb(244, 199, 58),
         qRgb(245, 197, 58),  qRgb(246, 195, 58),  qRgb(247, 193, 58), qRgb(248, 190, 57), qRgb(249, 188, 57),
         qRgb(250, 186, 57),  qRgb(251, 184, 56),  qRgb(251, 182, 55), qRgb(252, 179, 54), qRgb(252, 177, 54),
         qRgb(253, 174, 53),  qRgb(253, 172, 52),  qRgb(254, 169, 51), qRgb(254, 167, 50), qRgb(254, 164, 49),
         qRgb(254, 161, 48),  qRgb(254, 158, 47),  qRgb(254, 155, 45), qRgb(254, 153, 44), qRgb(254, 150, 43),
         qRgb(254, 147, 42),  qRgb(254, 144, 41),  qRgb(253, 141, 39), qRgb(253, 138, 38), qRgb(252, 135, 37),
         qRgb(252, 132, 35),  qRgb(251, 129, 34),  qRgb(251, 126, 33), qRgb(250, 123, 31), qRgb(249, 120, 30),
         qRgb(249, 117, 29),  qRgb(248, 114, 28),  qRgb(247, 111, 26), qRgb(246, 108, 25), qRgb(245, 105, 24),
         qRgb(244, 102, 23),  qRgb(243, 99, 21),   qRgb(242, 96, 20),  qRgb(241, 93, 19),  qRgb(240, 91, 18),
         qRgb(239, 88, 17),   qRgb(237, 85, 16),   qRgb(236, 83, 15),  qRgb(235, 80, 14),  qRgb(234, 78, 13),
         qRgb(232, 75, 12),   qRgb(231, 73, 12),   qRgb(229, 71, 11),  qRgb(228, 69, 10),  qRgb(226, 67, 10),
         qRgb(225, 65, 9),    qRgb(223, 63, 8),    qRgb(221, 61, 8),   qRgb(220, 59, 7),   qRgb(218, 57, 7),
         qRgb(216, 55, 6),    qRgb(214, 53, 6),    qRgb(212, 51, 5),   qRgb(210, 49, 5),   qRgb(208, 47, 5),
         qRgb(206, 45, 4),    qRgb(204, 43, 4),    qRgb(202, 42, 4),   qRgb(200, 40, 3),   qRgb(197, 38, 3),
         qRgb(195, 37, 3),    qRgb(193, 35, 2),    qRgb(190, 33, 2),   qRgb(188, 32, 2),   qRgb(185, 30, 2),
         qRgb(183, 29, 2),    qRgb(180, 27, 1),    qRgb(178, 26, 1),   qRgb(175, 24, 1),   qRgb(172, 23, 1),
         qRgb(169, 22, 1),    qRgb(167, 20, 1),    qRgb(164, 19, 1),   qRgb(161, 18, 1),   qRgb(158, 16, 1),
         qRgb(155, 15, 1),    qRgb(152, 14, 1),    qRgb(149, 13, 1),   qRgb(146, 11, 1),   qRgb(142, 10, 1),
         qRgb(139, 9, 2),     qRgb(136, 8, 2),     qRgb(133, 7, 2),    qRgb(129, 6, 2),    qRgb(126, 5, 2),
         qRgb(122, 4, 3),
     }}
};
